@@ -49,9 +49,11 @@
 
 package com.openexchange.imap;
 
-import java.net.SocketTimeoutException;
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
+import java.io.IOException;
 import java.nio.charset.UnsupportedCharsetException;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Date;
@@ -74,7 +76,9 @@ import javax.mail.internet.idn.IDNA;
 import javax.security.auth.Subject;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
+import com.openexchange.config.Reloadables;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.acl.ACLExtension;
 import com.openexchange.imap.acl.ACLExtensionInit;
@@ -104,6 +108,7 @@ import com.openexchange.log.audit.DefaultAttribute;
 import com.openexchange.log.audit.DefaultAttribute.Name;
 import com.openexchange.mail.MailExceptionCode;
 import com.openexchange.mail.Protocol;
+import com.openexchange.mail.api.AuthType;
 import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.IMailProperties;
@@ -119,11 +124,14 @@ import com.openexchange.mail.mime.MimeMailExceptionCode;
 import com.openexchange.mail.mime.MimeSessionPropertyNames;
 import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.mailaccount.MailAccountStorageService;
+import com.openexchange.net.ssl.SSLSocketFactoryProvider;
+import com.openexchange.net.ssl.config.SSLConfigurationService;
+import com.openexchange.net.ssl.exception.SSLExceptionCode;
+import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
 import com.openexchange.session.Sessions;
 import com.openexchange.timer.ScheduledTimerTask;
 import com.openexchange.timer.TimerService;
-import com.openexchange.tools.ssl.TrustAllSSLSocketFactory;
 import com.sun.mail.iap.ConnectQuotaExceededException;
 import com.sun.mail.iap.StarttlsRequiredException;
 import com.sun.mail.imap.GreetingListener;
@@ -146,8 +154,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     /**
      * The logger instance for {@link IMAPAccess} class.
      */
-    private static final transient org.slf4j.Logger LOG =
-        org.slf4j.LoggerFactory.getLogger(IMAPAccess.class);
+    private static final transient org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(IMAPAccess.class);
 
     /**
      * The max. temporary-down value; 5 Minutes.
@@ -172,6 +179,15 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
      */
     private static volatile Map<HostAndPort, Long> timedOutServers;
 
+    /**
+     * Gets the timedOutServers
+     *
+     * @return The timedOutServers
+     */
+    public static Map<HostAndPort, Long> getTimedOutServers() {
+        return timedOutServers;
+    }
+
     /**
      * Remembers whether a certain IMAP server supports the ACL extension.
      */
@@ -183,6 +199,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
     private static volatile ScheduledTimerTask cleanUpTimerTask;
 
     private static volatile Boolean checkConnectivityIfPolled;
+
     private static boolean checkConnectivityIfPolled() {
         Boolean b = checkConnectivityIfPolled;
         if (null == b) {
@@ -202,7 +219,6 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return b.booleanValue();
     }
 
-
     /*-
      * Member section
      */
@@ -299,8 +315,8 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             }
 
             @Override
-            public Map<String, String[]> getConfigFileNames() {
-                return null;
+            public Interests getInterests() {
+                return Reloadables.interestsForProperties("com.openexchange.imap.maxNumConnections");
             }
         });
     }
@@ -512,6 +528,24 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return tmp;
     }
 
+    @Override
+    protected boolean supports(AuthType authType) throws OXException {
+        switch (authType) {
+            case LOGIN:
+                return true;
+            case OAUTH:
+                try {
+                    IMAPConfig imapConfig = getIMAPConfig();
+                    final String serverUrl = new StringBuilder().append(imapConfig.getServer()).append(':').append(imapConfig.getPort()).toString();
+                    return IMAPCapabilityAndGreetingCache.getCapabilities(serverUrl, imapConfig.isSecure(), imapConfig.getIMAPProperties()).containsKey("AUTH=XOAUTH2");
+                } catch (IOException e) {
+                    throw MailExceptionCode.IO_ERROR.create(e, e.getMessage());
+                }
+            default:
+                return false;
+        }
+    }
+
     @Override
     public int getUnreadMessagesCount(final String fullname) throws OXException {
         if (!isConnected()) {
@@ -589,7 +623,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             javax.mail.Session imapSession;
             {
-                boolean forceSecure = imapConfig.isStartTls() || imapConfig.isRequireTls() || imapConfProps.isEnforceSecureConnection();
+                boolean forceSecure = imapConfig.isRequireTls() || imapConfProps.isEnforceSecureConnection();
                 imapSession = setConnectProperties(config, imapConfProps.getImapTimeout(), imapConfProps.getImapConnectionTimeout(), imapProps, JavaIMAPStore.class, forceSecure);
             }
             /*
@@ -625,6 +659,12 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 warnings.add(MailExceptionCode.PING_FAILED_AUTH.create(e, config.getServer(), config.getLogin()));
                 throw MimeMailException.handleMessagingException(e, config, session);
             } catch (final MessagingException e) {
+                if (MimeMailException.isSSLHandshakeException(e)) {
+                    OXException oxe = SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), config.getServer());
+                    warnings.add(oxe);
+                    throw oxe;
+                }
+
                 Exception cause = e.getNextException();
                 if (com.sun.mail.iap.ConnectionException.class.isInstance(cause)) {
                     OXException oxe = MimeMailException.handleMessagingException(e, config, session);
@@ -724,7 +764,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
              */
             {
                 final Class<? extends IMAPStore> clazz = useIMAPStoreCache() ? IMAPStoreCache.getInstance().getStoreClass() : JavaIMAPStore.class;
-                boolean forceSecure = accountId > 0 && (imapConfig.isStartTls() || imapConfig.isRequireTls() || imapConfProps.isEnforceSecureConnection());
+                boolean forceSecure = accountId > 0 && (imapConfig.isRequireTls() || imapConfProps.isEnforceSecureConnection());
                 imapSession = setConnectProperties(config, imapConfProps.getImapTimeout(), imapConfProps.getImapConnectionTimeout(), imapProps, clazz, forceSecure);
             }
             /*
@@ -762,18 +802,18 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 /*
                  * Check for a SocketTimeoutException
                  */
-                if (tmpDownEnabled) {
-                    final Exception nextException = e.getNextException();
-                    if (SocketTimeoutException.class.isInstance(nextException)) {
-                        /*
-                         * Remember a timed-out IMAP server on connect attempt
-                         */
-                        final Map<HostAndPort, Long> map = timedOutServers;
-                        if (null != map) {
-                            map.put(new HostAndPort(config.getServer(), config.getPort()), Long.valueOf(System.currentTimeMillis()));
-                        }
+                if (tmpDownEnabled && MimeMailException.isTimeoutException(e)) {
+                    /*
+                     * Remember a timed-out IMAP server on connect attempt
+                     */
+                    final Map<HostAndPort, Long> map = timedOutServers;
+                    if (null != map) {
+                        map.put(newHostAndPort(config), Long.valueOf(System.currentTimeMillis()));
                     }
                 }
+                if (MimeMailException.isSSLHandshakeException(e)) {
+                    throw SSLExceptionCode.UNTRUSTED_CERTIFICATE.create(e.getCause(), server);
+                }
                 {
                     Exception next = e.getNextException();
                     if (StarttlsRequiredException.class.isInstance(next)) {
@@ -1050,7 +1090,7 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         }
 
         MailConfig mailConfig = getMailConfig();
-        HostAndPort key = new HostAndPort(mailConfig.getServer(), mailConfig.getPort());
+        HostAndPort key = newHostAndPort(mailConfig);
         Long range = map.get(key);
         if (range != null) {
             if (System.currentTimeMillis() - range.longValue() <= imapConfProps.getImapTemporaryDown()) {
@@ -1236,15 +1276,41 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return true;
     }
 
-    private static final class HostAndPort {
+    /**
+     * Creates a new <code>HostAndPort</code> instance from given arguments.
+     *
+     * @param config The configuration providing host name or IP address of the IMAP server as well as port
+     * @return The new instance
+     */
+    public static HostAndPort newHostAndPort(MailConfig config) {
+        return newHostAndPort(config.getServer(), config.getPort());
+    }
+
+    /**
+     * Creates a new <code>HostAndPort</code> instance from given arguments.
+     *
+     * @param host The host name or IP address of the IMAP server
+     * @param port The port
+     * @return The new instance
+     */
+    public static HostAndPort newHostAndPort(String host, int port) {
+        return new HostAndPort(host, port);
+    }
+
+    /** Simple class to hold host and port for an IMAP end-point */
+    public static final class HostAndPort {
 
         private final String host;
-
         private final int port;
-
         private final int hashCode;
 
-        public HostAndPort(final String host, final int port) {
+        /**
+         * Initializes a new {@link HostAndPort}.
+         *
+         * @param host The host name or IP address of the IMAP server
+         * @param port The port
+         */
+        HostAndPort(final String host, final int port) {
             super();
             if (port < 0 || port > 0xFFFF) {
                 throw new IllegalArgumentException("port out of range:" + port);
@@ -1316,6 +1382,12 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
          * Specify NOOP behavior
          */
         imapProps.put("mail.imap.issueNoopToKeepConnectionAlive", "false");
+        /*
+         * Whether to extend (default) or to overwrite pre-login capabilities
+         */
+        if (config.getIMAPProperties().isOverwritePreLoginCapabilities()) {
+            imapProps.put("mail.imap.overwriteprelogincapabilities", "true");
+        }
         /*
          * Enable/disable audit log
          */
@@ -1331,11 +1403,30 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
                 imapProps.put("mail.imap.greeting.listeners", Collections.singletonList(greetingListener));
             }
         }
+        /*
+         * Allow round-robin address election for primary IMAP account
+         */
+        if (config.getAccountId() == MailAccount.DEFAULT_ID) {
+            boolean useMultipleAddresses = IMAPProperties.getInstance().isUseMultipleAddresses();
+            if (useMultipleAddresses) {
+                imapProps.put("mail.imap.multiAddress.enabled", "true");
+            }
+        }
+        /*
+         * Enable XOAUTH2/OAUTHBEARER (if appropriate)
+         */
+        if (AuthType.OAUTH == config.getAuthType()) {
+            imapProps.put("mail.imap.auth.mechanisms", "XOAUTH2");
+        }
         /*
          * Check if a secure IMAP connection should be established
          */
-        final String sPort = String.valueOf(config.getPort());
-        final String socketFactoryClass = TrustAllSSLSocketFactory.class.getName();
+        String sPort = String.valueOf(config.getPort());
+        SSLSocketFactoryProvider factoryProvider = Services.getService(SSLSocketFactoryProvider.class);
+        String socketFactoryClass = factoryProvider.getDefault().getClass().getName();
+        String protocols = config.getIMAPProperties().getSSLProtocols();
+        String cipherSuites = config.getIMAPProperties().getSSLCipherSuites();
+        SSLConfigurationService sslConfigService = Services.getService(SSLConfigurationService.class);
         if (config.isSecure()) {
             /*
              * Enables the use of the STARTTLS command.
@@ -1354,13 +1445,25 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             /*
              * Specify SSL protocols
              */
-            imapProps.put("mail.imap.ssl.protocols", config.getIMAPProperties().getSSLProtocols());
+            if (Strings.isNotEmpty(protocols)) {
+                imapProps.put("mail.imap.ssl.protocols", protocols);
+            } else {
+                if (sslConfigService == null) {
+                    throw ServiceExceptionCode.absentService(SSLConfigurationService.class);
+                }
+                imapProps.put("mail.imap.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+            }
             /*
              * Specify SSL cipher suites
              */
-            final String cipherSuites = config.getIMAPProperties().getSSLCipherSuites();
-            if (false == Strings.isEmpty(cipherSuites)) {
+
+            if (Strings.isNotEmpty(cipherSuites)) {
                 imapProps.put("mail.imap.ssl.ciphersuites", cipherSuites);
+            } else {
+                if (null == sslConfigService) {
+                    throw ServiceExceptionCode.absentService(SSLConfigurationService.class);
+                }
+                imapProps.put("mail.imap.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
             }
         } else {
             /*
@@ -1382,13 +1485,24 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
             /*
              * Specify SSL protocols
              */
-            imapProps.put("mail.imap.ssl.protocols", config.getIMAPProperties().getSSLProtocols());
+            if (Strings.isNotEmpty(protocols)) {
+                imapProps.put("mail.imap.ssl.protocols", protocols);
+            } else {
+                if (null == sslConfigService) {
+                    throw ServiceExceptionCode.absentService(SSLConfigurationService.class);
+                }
+                imapProps.put("mail.imap.ssl.protocols", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedProtocols()));
+            }
             /*
              * Specify SSL cipher suites
              */
-            final String cipherSuites = config.getIMAPProperties().getSSLCipherSuites();
-            if (false == Strings.isEmpty(cipherSuites)) {
+            if (Strings.isNotEmpty(cipherSuites)) {
                 imapProps.put("mail.imap.ssl.ciphersuites", cipherSuites);
+            } else {
+                if (null == sslConfigService) {
+                    throw ServiceExceptionCode.absentService(SSLConfigurationService.class);
+                }
+                imapProps.put("mail.imap.ssl.ciphersuites", Strings.toWhitespaceSeparatedList(sslConfigService.getSupportedCipherSuites()));
             }
             // imapProps.put("mail.imap.ssl.enable", "true");
             /*
@@ -1404,6 +1518,27 @@ public final class IMAPAccess extends MailAccess<IMAPFolderStorage, IMAPMessageS
         return imapSession;
     }
 
+    private static int getHashFor(int user, int context) {
+        try {
+            MessageDigest md = MessageDigest.getInstance("MD5");
+
+            md.update((byte) user);
+            md.update((byte) context);
+
+            // Calculate hash & create its string representation
+            byte[] bytes = md.digest();
+
+            int i = 0;
+            for (byte b : bytes) {
+                i += b;
+            }
+            return (i < 0) ? -i : i;
+        } catch (NoSuchAlgorithmException e) {
+            // Ignore
+        }
+        return 0;
+    }
+
     @Override
     public String toString() {
         IMAPStore imapStore = this.imapStore;
