@@ -50,7 +50,10 @@
 package com.openexchange.imap;
 
 import static com.openexchange.imap.threader.Threadables.applyThreaderTo;
+import static com.openexchange.imap.threadsort.ThreadSortUtil.toUnifiedThreadResponse;
 import static com.openexchange.mail.mime.utils.MimeStorageUtility.getFetchProfile;
+import java.io.IOException;
+import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
@@ -64,8 +67,11 @@ import java.util.Set;
 import javax.mail.FetchProfile;
 import javax.mail.Message;
 import javax.mail.MessagingException;
+import javax.mail.search.SearchException;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
+import com.openexchange.config.Reloadables;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.command.MailMessageFillerIMAPCommand;
 import com.openexchange.imap.command.MessageFetchIMAPCommand;
@@ -78,6 +84,7 @@ import com.openexchange.imap.threader.Threadables;
 import com.openexchange.imap.threader.references.Conversation;
 import com.openexchange.imap.threader.references.ConversationCache;
 import com.openexchange.imap.threader.references.Conversations;
+import com.openexchange.imap.threadsort.MailThreadParser;
 import com.openexchange.imap.threadsort.MessageInfo;
 import com.openexchange.imap.threadsort.ThreadSortNode;
 import com.openexchange.imap.threadsort.ThreadSortUtil;
@@ -92,6 +99,7 @@ import com.openexchange.mail.api.IMailFolderStorage;
 import com.openexchange.mail.api.IMailMessageStorage;
 import com.openexchange.mail.api.MailAccess;
 import com.openexchange.mail.dataobjects.MailMessage;
+import com.openexchange.mail.dataobjects.MailThread;
 import com.openexchange.mail.dataobjects.ThreadSortMailMessage;
 import com.openexchange.mail.mime.ExtendedMimeMessage;
 import com.openexchange.mail.search.SearchTerm;
@@ -100,8 +108,15 @@ import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.session.Session;
 import com.openexchange.threadpool.AbstractTask;
 import com.openexchange.threadpool.ThreadPools;
+import com.sun.mail.iap.Argument;
+import com.sun.mail.iap.BadCommandException;
+import com.sun.mail.iap.ProtocolException;
+import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.IMAPStore;
+import com.sun.mail.imap.protocol.IMAPProtocol;
+import com.sun.mail.imap.protocol.IMAPResponse;
+import com.sun.mail.imap.protocol.SearchSequence;
 import gnu.trove.map.TIntObjectMap;
 import gnu.trove.map.TLongObjectMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
@@ -142,8 +157,8 @@ public final class IMAPConversationWorker {
             }
 
             @Override
-            public Map<String, String[]> getConfigFileNames() {
-                return null;
+            public Interests getInterests() {
+                return Reloadables.interestsForProperties("com.openexchange.imap.useImapThreaderIfSupported");
             }
         });
     }
@@ -186,7 +201,7 @@ public final class IMAPConversationWorker {
      * @return The message conversations
      * @throws OXException If message conversations cannot be returned
      */
-    public List<List<MailMessage>> getThreadSortedMessages(final String fullName, final boolean includeSent, final boolean cache, final IndexRange indexRange, final long max, final MailSortField sortField, final OrderDirection order, final MailField[] mailFields, String[] headerNames) throws OXException {
+    public List<List<MailMessage>> getThreadSortedMessages(final String fullName, final boolean includeSent, final boolean cache, final IndexRange indexRange, final long max, final MailSortField sortField, final OrderDirection order, final MailField[] mailFields, String[] headerNames, SearchTerm<?> searchTerm) throws OXException {
         IMAPFolder sentFolder = null;
         try {
             final String sentFullName = imapFolderStorage.getSentFolder();
@@ -196,9 +211,7 @@ public final class IMAPConversationWorker {
                 return Collections.emptyList();
             }
             int lookAhead;
-            if (max <= 0) {
-                lookAhead = -1;
-            } else {
+            {
                 lookAhead = 1000;
                 if (null != indexRange) {
                     while (indexRange.end >= (lookAhead / 2)) {
@@ -223,10 +236,10 @@ public final class IMAPConversationWorker {
                 throw MailExceptionCode.ILLEGAL_ARGUMENT.create();
             }
             if (useImapThreaderIfSupported() && imapMessageStorage.getImapConfig().getImapCapabilities().hasThreadReferences()) {
-                return doImapThreadSort(fullName, indexRange, sortField, order, sentFullName, messageCount, lookAhead, mergeWithSent, mailFields, headerNames);
+                return doImapThreadSort(fullName, indexRange, sortField, order, sentFullName, messageCount, lookAhead, mergeWithSent, mailFields, headerNames, searchTerm);
             }
             // Use built-in algorithm
-            return doReferenceOnlyThreadSort(fullName, indexRange, sortField, order, sentFullName, lookAhead, mergeWithSent, mailFields, headerNames);
+            return doReferenceOnlyThreadSort(fullName, indexRange, sortField, order, sentFullName, lookAhead, mergeWithSent, mailFields, headerNames, searchTerm);
         } catch (final MessagingException e) {
             throw imapMessageStorage.handleMessagingException(fullName, e);
         } catch (final RuntimeException e) {
@@ -239,8 +252,19 @@ public final class IMAPConversationWorker {
 
     private static final MailFields FIELDS_FLAGS = new MailFields(MailField.FLAGS, MailField.COLOR_LABEL);
 
-    private List<List<MailMessage>> doReferenceOnlyThreadSort(final String fullName, IndexRange indexRange, MailSortField sortField, OrderDirection order, final String sentFullName, int lookAhead, final boolean mergeWithSent, MailField[] mailFields, final String[] headerNames) throws MessagingException, OXException {
+    private List<List<MailMessage>> doReferenceOnlyThreadSort(final String fullName, IndexRange indexRange, MailSortField sortField, OrderDirection order, final String sentFullName, int lookAhead, final boolean mergeWithSent, MailField[] mailFields, final String[] headerNames, SearchTerm<?> searchTerm) throws MessagingException, OXException {
+        boolean useSearchTerm = searchTerm != null;
         final MailFields usedFields = new MailFields(mailFields);
+        // add necessary fields for searchTerm
+        if (searchTerm != null) {
+            List<MailField> searchFields = new ArrayList<>();
+            searchTerm.addMailField(searchFields);
+            for (MailField field : searchFields) {
+                if (!usedFields.contains(field)) {
+                    usedFields.add(field);
+                }
+            }
+        }
         usedFields.add(MailField.THREAD_LEVEL);
         usedFields.add(MailField.RECEIVED_DATE);
         usedFields.add(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
@@ -272,6 +296,17 @@ public final class IMAPConversationWorker {
                 String argsHash = ConversationCache.getArgsHash(sortField, order, lookAhead, mergeWithSent, usedFields, headerNames, total, uidNext, sentTotal, sentUidNext);
                 List<List<MailMessage>> list = conversationCache.getCachedConversations(fullName, imapMessageStorage.getAccountId(), argsHash, imapMessageStorage.getSession());
                 if (null != list) {
+
+                    // Filter for searchterm
+                    if (useSearchTerm) {
+                        List<List<MailMessage>> result = filterThreads(list, searchTerm);
+                        // Slice & fill with recent flags
+                        if (usedFields.containsAny(FIELDS_FLAGS)) {
+                            return sliceAndFill(result, fullName, indexRange, sentFullName, mergeWithSent, FIELDS_FLAGS, null, body, isRev1);
+                        }
+                        return sliceMessages(result, indexRange);
+                    }
+
                     // Slice & fill with recent flags
                     if (usedFields.containsAny(FIELDS_FLAGS)) {
                         return sliceAndFill(list, fullName, indexRange, sentFullName, mergeWithSent, FIELDS_FLAGS, null, body, isRev1);
@@ -314,7 +349,7 @@ public final class IMAPConversationWorker {
                 if (false == sentMessages.isEmpty()) {
                     // Filter messages already contained in conversations
                     {
-                        Set<String> allMessageIds = new HashSet<String>(conversations.size());
+                        Set<String> allMessageIds = new HashSet<>(conversations.size());
                         for (final Conversation conversation : conversations) {
                             conversation.addMessageIdsTo(allMessageIds);
                         }
@@ -322,7 +357,7 @@ public final class IMAPConversationWorker {
                         for (MailMessage sentMessage : sentMessages) {
                             if (false == allMessageIds.contains(sentMessage.getMessageId())) {
                                 if (null == tmp) {
-                                    tmp = new LinkedList<MailMessage>();
+                                    tmp = new LinkedList<>();
                                 }
                                 tmp.add(sentMessage);
                             }
@@ -353,11 +388,12 @@ public final class IMAPConversationWorker {
         // Comparator
         MailMessageComparator threadComparator = COMPARATOR_DESC;
         // Sort
-        final List<List<MailMessage>> list = new LinkedList<List<MailMessage>>();
+        final List<List<MailMessage>> list = new LinkedList<>();
         for (final Conversation conversation : conversations) {
             list.add(conversation.getMessages(threadComparator));
         }
         conversations = null;
+
         // Sort root elements
         {
             MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
@@ -367,6 +403,11 @@ public final class IMAPConversationWorker {
         // Slice & fill
         if (body || (lookAhead > CONVERSATION_CACHE_THRESHOLD)) {
             // Body requested - Do not cache at all
+            // Filter for searchterm
+            if (useSearchTerm) {
+                fillMessages(list, fullName, sentFullName, mergeWithSent, usedFields, headerNames, body, isRev1);
+                return filterThreads(list, searchTerm);
+            }
             return sliceAndFill(list, fullName, indexRange, sentFullName, mergeWithSent, usedFields, headerNames, body, isRev1);
         }
 
@@ -374,8 +415,15 @@ public final class IMAPConversationWorker {
         if (null == indexRange) {
             // Fill (except flags)
             fillMessages(list, fullName, sentFullName, mergeWithSent, usedFields, headerNames, body, isRev1);
+
             // Put into cache
             conversationCache.putCachedConversations(list, fullName, imapMessageStorage.getAccountId(), argsHash, imapMessageStorage.getSession());
+
+            // Filter for searchterm
+            if (useSearchTerm) {
+                return filterThreads(list, searchTerm);
+            }
+
             // All
             return list;
         }
@@ -450,6 +498,23 @@ public final class IMAPConversationWorker {
         return slice;
     }
 
+    private List<List<MailMessage>> filterThreads(List<List<MailMessage>> list, SearchTerm<?> searchTerm) throws OXException {
+        List<List<MailMessage>> result = new ArrayList<>();
+        for (List<MailMessage> messages : list) {
+            boolean containsFlag = false;
+            for (MailMessage message : messages) {
+                if (searchTerm.matches(message)) {
+                    containsFlag = true;
+                    break;
+                }
+            }
+            if (containsFlag) {
+                result.add(messages);
+            }
+        }
+        return result;
+    }
+
     private List<List<MailMessage>> sliceAndFill(List<List<MailMessage>> listOfConversations, String fullName, IndexRange indexRange, String sentFullName, boolean mergeWithSent, MailFields usedFields, String[] headerNames, boolean body, boolean isRev1) throws MessagingException, OXException {
         // Check for index range
         List<List<MailMessage>> list = sliceMessages(listOfConversations, indexRange);
@@ -509,8 +574,8 @@ public final class IMAPConversationWorker {
         try {
             if (mergeWithSent) {
                 FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
-                List<MailMessage> msgs = new LinkedList<MailMessage>();
-                List<MailMessage> sentmsgs = new LinkedList<MailMessage>();
+                List<MailMessage> msgs = new LinkedList<>();
+                List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     for (MailMessage m : conversation) {
                         if (sentFullName.equals(m.getFolder())) {
@@ -529,7 +594,7 @@ public final class IMAPConversationWorker {
                     new MailMessageFillerIMAPCommand(sentmsgs, isRev1, fetchProfile, imapServerInfo, imapFolder).doCommand();
                 }
             } else {
-                List<MailMessage> msgs = new LinkedList<MailMessage>();
+                List<MailMessage> msgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
@@ -548,8 +613,8 @@ public final class IMAPConversationWorker {
         // Fill messages
         if (mergeWithSent) {
             FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
-            List<MailMessage> msgs = new LinkedList<MailMessage>();
-            List<MailMessage> sentmsgs = new LinkedList<MailMessage>();
+            List<MailMessage> msgs = new LinkedList<>();
+            List<MailMessage> sentmsgs = new LinkedList<>();
             for (List<MailMessage> conversation : list) {
                 for (MailMessage m : conversation) {
                     if (sentFullName.equals(m.getFolder())) {
@@ -569,9 +634,9 @@ public final class IMAPConversationWorker {
             }
         } else {
             if (body) {
-                List<List<MailMessage>> newlist = new LinkedList<List<MailMessage>>();
+                List<List<MailMessage>> newlist = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
-                    List<MailMessage> newconversation = new LinkedList<MailMessage>();
+                    List<MailMessage> newconversation = new LinkedList<>();
                     for (MailMessage mailMessage : conversation) {
                         newconversation.add(imapMessageStorage.getMessage(mailMessage.getFolder(), mailMessage.getMailId(), false));
                     }
@@ -579,7 +644,7 @@ public final class IMAPConversationWorker {
                 }
                 list = newlist;
             } else {
-                List<MailMessage> msgs = new LinkedList<MailMessage>();
+                List<MailMessage> msgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
@@ -592,7 +657,7 @@ public final class IMAPConversationWorker {
         imapMessageStorage.setAccountInfo2(list);
     }
 
-    private List<List<MailMessage>> doImapThreadSort(String fullName, IndexRange indexRange, MailSortField sortField, OrderDirection order, String sentFullName, int messageCount, int lookAhead, boolean mergeWithSent, MailField[] mailFields, String[] headerNames) throws OXException, MessagingException {
+    private List<List<MailMessage>> doImapThreadSort(String fullName, IndexRange indexRange, MailSortField sortField, OrderDirection order, String sentFullName, int messageCount, int lookAhead, boolean mergeWithSent, MailField[] mailFields, String[] headerNames, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         // Parse THREAD response to a list structure
 
         final MailFields usedFields = new MailFields(mailFields);
@@ -614,7 +679,7 @@ public final class IMAPConversationWorker {
             } else {
                 fp = Conversations.getFetchProfileConversationByHeaders(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
-            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo());
+            List<Conversation> conversations = ThreadSorts.getConversationList(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, fp, imapMessageStorage.getImapServerInfo(), searchTerm);
             // Merge with sent folder
             {
                 // Switch folder
@@ -632,7 +697,7 @@ public final class IMAPConversationWorker {
             }
             final MailMessageComparator threadComparator = COMPARATOR_DESC;
             // Sort
-            list = new LinkedList<List<MailMessage>>();
+            list = new LinkedList<>();
             for (final Conversation conversation : conversations) {
                 list.add(conversation.getMessages(threadComparator));
             }
@@ -661,8 +726,8 @@ public final class IMAPConversationWorker {
             // Fill selected chunk
             if (!list.isEmpty()) {
                 FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
-                List<MailMessage> msgs = new LinkedList<MailMessage>();
-                List<MailMessage> sentmsgs = new LinkedList<MailMessage>();
+                List<MailMessage> msgs = new LinkedList<>();
+                List<MailMessage> sentmsgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     for (MailMessage m : conversation) {
                         if (mergeWithSent && sentFullName.equals(m.getFolder())) {
@@ -680,7 +745,7 @@ public final class IMAPConversationWorker {
                 }
             }
         } else {
-            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, imapMessageStorage.getImapServerInfo(), null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
+            list = ThreadSorts.getConversations(imapMessageStorage.getImapFolder(), getSortRange(lookAhead, messageCount, order), isRev1, imapMessageStorage.getImapServerInfo(), searchTerm, null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             // Sort root elements
             {
                 final MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
@@ -704,9 +769,9 @@ public final class IMAPConversationWorker {
             }
             // Fill selected chunk
             if (body) {
-                List<List<MailMessage>> newlist = new LinkedList<List<MailMessage>>();
+                List<List<MailMessage>> newlist = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
-                    List<MailMessage> newconversation = new LinkedList<MailMessage>();
+                    List<MailMessage> newconversation = new LinkedList<>();
                     for (MailMessage mailMessage : conversation) {
                         newconversation.add(imapMessageStorage.getMessage(mailMessage.getFolder(), mailMessage.getMailId(), false));
                     }
@@ -714,7 +779,7 @@ public final class IMAPConversationWorker {
                 }
                 list = newlist;
             } else {
-                List<MailMessage> msgs = new LinkedList<MailMessage>();
+                List<MailMessage> msgs = new LinkedList<>();
                 for (List<MailMessage> conversation : list) {
                     msgs.addAll(conversation);
                 }
@@ -780,6 +845,224 @@ public final class IMAPConversationWorker {
 
     // -----------------------------------------------------------------------------------------------------------------------------------
 
+    /**
+     * Performs the IMAP THREAD REFERENCE command.
+     *
+     * @param fullName The full name
+     * @param size The number of recent/latest messages
+     * @param sortField The sort field (for root elements)
+     * @param order The sort order
+     * @param searchTerm The optional search term to apply
+     * @param mailFields The mail fields to query
+     * @param headerNames The optional header names to query
+     * @return The mail threads
+     * @throws OXException If mail threads cannot be returned
+     */
+    public List<MailThread> getThreadReferences(String fullName, int size, MailSortField sortField, OrderDirection order, SearchTerm<?> searchTerm, MailField[] mailFields, String[] headerNames) throws OXException {
+        if (0 == size) {
+            return Collections.emptyList();
+        }
+
+        try {
+            imapMessageStorage.openReadOnly(fullName);
+            int messageCount = imapMessageStorage.getImapFolder().getMessageCount();
+            if (0 >= messageCount) {
+                return Collections.emptyList();
+            }
+
+            // Build-up effective fields
+            MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
+            MailFields usedFields = new MailFields();
+            IMAPMessageStorage.prepareMailFieldsForVirtualFolder(usedFields, fullName, imapMessageStorage.getSession());
+            usedFields.addAll(mailFields);
+            usedFields.add(MailField.toField(effectiveSortField.getListField()));
+
+            // Deny body
+            boolean body = usedFields.contains(MailField.BODY) || usedFields.contains(MailField.FULL);
+            if (body) {
+                throw MailExceptionCode.UNSUPPORTED_OPERATION.create();
+            }
+
+            // Compose search term
+            javax.mail.search.SearchTerm jmsSearchTerm;
+            if (searchTerm == null) {
+                jmsSearchTerm = null;
+            } else {
+                if (searchTerm.containsWildcard()) {
+                    jmsSearchTerm = searchTerm.getNonWildcardJavaMailSearchTerm();
+                } else {
+                    jmsSearchTerm = searchTerm.getJavaMailSearchTerm();
+                }
+            }
+
+            // Define heading search keys
+            int numMsgs;
+            String[] searchKeys;
+            if (size < 0 || size >= messageCount) {
+                searchKeys = null;
+                numMsgs = messageCount;
+            } else {
+                int start = messageCount - size + 1;
+                searchKeys = new String[] { new StringBuilder(16).append(start).append(':').append(messageCount).toString() };
+                numMsgs = size;
+            }
+
+            // Issue command
+            String threadList = (String) imapMessageStorage.getImapFolder().doCommand(new ThreadReferencesProtocolCommand(true, searchKeys, jmsSearchTerm));
+            if (null == threadList) {
+                return Collections.emptyList();
+            }
+
+            // Parse unified THREAD response
+            return parseThreadList(toUnifiedThreadResponse(threadList), fullName, numMsgs, sortField, order, usedFields, headerNames);
+        } catch (final MessagingException e) {
+            throw imapMessageStorage.handleMessagingException(fullName, e);
+        } catch (final RuntimeException e) {
+            throw imapMessageStorage.handleRuntimeException(e);
+        } finally {
+            IMAPFolderWorker.clearCache(imapMessageStorage.getImapFolder());
+        }
+    }
+
+    private static final class ThreadReferencesProtocolCommand implements IMAPFolder.ProtocolCommand {
+
+        private final String[] searchKeys;
+        private final Argument searchSequence;
+        private final boolean uid;
+
+        /**
+         * Initializes a new {@link IMAPConversationWorker.ThreadReferencesProtocolCommand}.
+         *
+         * @throws MessagingException If initialization fails
+         */
+        ThreadReferencesProtocolCommand(boolean uid, String[] searchKeys, javax.mail.search.SearchTerm searchTerm) throws MessagingException {
+            super();
+            this.uid = uid;
+            this.searchKeys = (searchKeys == null || searchKeys.length == 0) ? new String[] {"ALL"} : searchKeys;
+            if (null == searchTerm) {
+                this.searchSequence = null;
+            } else {
+                try {
+                    searchSequence = new SearchSequence().generateSequence(searchTerm, "UTF-8");
+                } catch (IOException ioex) {
+                    // should never happen
+                    throw new SearchException(ioex.toString(), ioex);
+                }
+            }
+        }
+
+        @Override
+        public Object doCommand(IMAPProtocol protocol) throws ProtocolException {
+            if (!protocol.hasCapability("THREAD=REFERENCES")) {
+                throw new BadCommandException("THREAD=REFERENCES not supported");
+            }
+            if (searchKeys == null || searchKeys.length == 0) {
+                throw new BadCommandException("Must have at least one sort term");
+            }
+
+            // UID THREAD REFERENCES UTF-8 1:1000 BODY "Hello"
+
+            Argument args = new Argument();
+
+            args.writeAtom("REFERENCES");    // charset specification
+            args.writeAtom("UTF-8");    // charset specification
+
+            // Add (sort) terms
+            {
+                Argument sargs = new Argument();
+                for (int i = 0; i < searchKeys.length; i++) {
+                    sargs.writeAtom(searchKeys[i]);
+                }
+                args.writeArgument(sargs);  // sort criteria
+            }
+
+            if (searchSequence != null) {
+                args.append(searchSequence);
+            }
+
+            Response[] r = protocol.command(uid ? "UID THREAD" : "THREAD", args);
+            Response response = r[r.length - 1];
+            String result = null;
+
+            // Grab all THREAD responses
+            if (response.isOK()) { // command successful
+                String threadStr = "THREAD";
+                for (int i = 0, len = r.length - 1; i < len; i++) {
+                    if (!(r[i] instanceof IMAPResponse)) {
+                        continue;
+                    }
+                    IMAPResponse ir = (IMAPResponse) r[i];
+                    if (ir.keyEquals(threadStr)) {
+                        result = ir.getRest();
+                        r[i] = null;
+                    }
+                }
+
+                // dispatch remaining untagged responses
+                protocol.notifyResponseHandlers(r);
+            } else {
+                // dispatch remaining untagged responses
+                protocol.notifyResponseHandlers(r);
+                protocol.handleResult(response);
+            }
+
+            return result;
+        }
+    }
+
+    private List<MailThread> parseThreadList(String unifiedThreadList, String fullName, int numMsgs, MailSortField sortField, OrderDirection order, MailFields usedFields, String[] headerNames) throws OXException, MessagingException {
+        // Parse the unified THREAD=REFERENCES response
+        MailThreadParser.ParseResult parseResult = MailThreadParser.getInstance().parseUnifiedResponse(unifiedThreadList, fullName, numMsgs);
+
+        // Fill requested fields/headers
+        {
+            FetchProfile fetchProfile = IMAPMessageStorage.checkFetchProfile(getFetchProfile(usedFields.toArray(), headerNames, null, null, true));
+            boolean isRev1 = imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1();
+            TLongObjectMap<MailMessage> messages = parseResult.getMessages();
+            new MailMessageFillerIMAPCommand(messages.valueCollection(), isRev1, fetchProfile, imapMessageStorage.getImapServerInfo(), imapMessageStorage.getImapFolder()).doCommand();
+            imapMessageStorage.setAccountInfo(messages.values(new MailMessage[messages.size()]));
+        }
+
+        // Sort & return
+        List<MailThread> mailThreads = parseResult.getMailThreads();
+        {
+            final MailSortField effectiveSortField = null == sortField ? MailSortField.RECEIVED_DATE : sortField;
+            final Comparator<MailThread> threadComparator = getThreadComparator(effectiveSortField, order, imapMessageStorage.getLocale());
+            Collections.sort(mailThreads, threadComparator);
+        }
+        return mailThreads;
+    }
+
+    private Comparator<MailThread> getThreadComparator(final MailSortField sortField, final OrderDirection order, Locale locale) {
+        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale);
+        Comparator<MailThread> threadComparator = new Comparator<MailThread>() {
+
+            @Override
+            public int compare(MailThread o1, MailThread o2) {
+                MailMessage msg1 = o1.getParent();
+                MailMessage msg2 = o2.getParent();
+
+                int result = comparator.compare(msg1, msg2);
+                if ((0 != result) || (MailSortField.RECEIVED_DATE != sortField)) {
+                    return result;
+                }
+
+                // Zero as comparison result AND primarily sorted by received-date
+                String inReplyTo1 = msg1.getInReplyTo();
+                String inReplyTo2 = msg2.getInReplyTo();
+                if (null == inReplyTo1) {
+                    result = null == inReplyTo2 ? 0 : -1;
+                } else {
+                    result = null == inReplyTo2 ? 1 : 0;
+                }
+                return 0 == result ? new MailMessageComparator(MailSortField.SENT_DATE, OrderDirection.DESC.equals(order), null).compare(msg1, msg2) : result;
+            }
+        };
+        return threadComparator;
+    }
+
+    // -----------------------------------------------------------------------------------------------------------------------------------
+
     public MailMessage[] getThreadSortedMessages(final String fullName, final IndexRange indexRange, final MailSortField sortField, final OrderDirection order, final SearchTerm<?> searchTerm, final MailField[] mailFields) throws OXException {
         try {
             imapMessageStorage.openReadOnly(fullName);
@@ -846,7 +1129,7 @@ public final class IMAPConversationWorker {
                 return imapMessageStorage.getAllMessages(fullName, indexRange, sortField, order, mailFields);
             }
             final List<MessageInfo> messageIds = ThreadSortUtil.fromThreadResponse(threadList);
-            final TIntObjectMap<MessageInfo> seqNum2MessageId = new TIntObjectHashMap<MessageInfo>(messageIds.size());
+            final TIntObjectMap<MessageInfo> seqNum2MessageId = new TIntObjectHashMap<>(messageIds.size());
             for (final MessageInfo messageId : messageIds) {
                 seqNum2MessageId.put(messageId.getMessageNumber(), messageId);
             }
@@ -866,7 +1149,7 @@ public final class IMAPConversationWorker {
                 final Map<MessageInfo, MailMessage> mapping;
                 {
                     TLongObjectMap<MailMessage> messages = new SimpleFetchIMAPCommand(imapMessageStorage.getImapFolder(), imapMessageStorage.getImapConfig().getImapCapabilities().hasIMAP4rev1(), MessageInfo.toSeqNums(messageIds).toArray(), fetchProfile, imapMessageStorage.getImapServerInfo()).doCommand();
-                    mapping = new HashMap<MessageInfo, MailMessage>(messages.size());
+                    mapping = new HashMap<>(messages.size());
                     messages.forEachEntry(new TLongObjectProcedure<MailMessage>() {
 
                         @Override
@@ -884,13 +1167,31 @@ public final class IMAPConversationWorker {
                 /*
                  * Output as flat list
                  */
-                final List<MailMessage> flatList = new LinkedList<MailMessage>();
+                List<MailMessage> flatList = new LinkedList<>();
                 if (usedFields.contains(MailField.ACCOUNT_NAME) || usedFields.contains(MailField.FULL)) {
                     for (final MailMessage mail : flatList) {
                         imapMessageStorage.setAccountInfo(mail);
                     }
                 }
                 ThreadSortUtil.toFlatList(structuredList, flatList);
+                /*
+                 * Apply index range (if any)
+                 */
+                if (indexRange != null) {
+                    int fromIndex = indexRange.start;
+                    int toIndex = indexRange.end;
+                    int size = flatList.size();
+                    if (size == 0 || fromIndex >= size) {
+                        return IMailMessageStorage.EMPTY_RETVAL;
+                    }
+                    /*
+                     * Reset end index if out of range
+                     */
+                    if (toIndex >= size) {
+                        toIndex = size;
+                    }
+                    flatList = flatList.subList(fromIndex, toIndex);
+                }
                 return flatList.toArray(new MailMessage[flatList.size()]);
             }
             /*
@@ -947,7 +1248,7 @@ public final class IMAPConversationWorker {
             /*
              * Output as flat list
              */
-            final List<MailMessage> flatList = new LinkedList<MailMessage>();
+            final List<MailMessage> flatList = new LinkedList<>();
             ThreadSortUtil.toFlatList(structuredList, flatList);
             return flatList.toArray(new MailMessage[flatList.size()]);
         } catch (final MessagingException e) {
