@@ -52,10 +52,10 @@ package com.openexchange.imap;
 import static com.openexchange.java.Strings.isEmpty;
 import gnu.trove.list.TIntList;
 import gnu.trove.list.array.TIntArrayList;
-import gnu.trove.list.linked.TIntLinkedList;
 import gnu.trove.map.TIntObjectMap;
 import gnu.trove.map.hash.TIntObjectHashMap;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
@@ -511,7 +511,7 @@ public class IMAPDefaultFolderChecker {
      */
     protected TIntObjectMap<String> getSpecialUseInfo(String[] names, String[] fullNames) throws OXException {
         try {
-            TIntObjectMap<String> indexes = new TIntObjectHashMap<String>(6);
+            TIntObjectMap<String> indexes = new TIntObjectHashMap<>(6);
             IMAPFolder imapFolder = (IMAPFolder) imapStore.getFolder(INBOX);
             boolean isPrimary = MailAccount.DEFAULT_ID == accountId;
 
@@ -589,35 +589,36 @@ public class IMAPDefaultFolderChecker {
         // Get default folders names and full names
         DefaultFolderNamesProvider defaultFolderNamesProvider = new DefaultFolderNamesProvider(accountId, session.getUserId(), session.getContextId());
         String[] fullNames = defaultFolderNamesProvider.getDefaultFolderFullnames(imapConfig, isSpamOptionEnabled);
-        String[] names = defaultFolderNamesProvider.getDefaultFolderNames(imapConfig, isSpamOptionEnabled);
+        String[] names = Arrays.copyOfRange(imapConfig.getStandardNames(), 0, isSpamOptionEnabled ? 6 : 4);
+        UserSettingMail usm = UserSettingMailStorage.getInstance().getUserSettingMail(session.getUserId(), session.getContextId());
+        String[] defaultNames = defaultFolderNamesProvider.getDefaultFolderNames(usm.getStdTrashName(), usm.getStdSentName(), usm.getStdDraftsName(), usm.getStdSpamName(), usm.getConfirmedSpam(), usm.getConfirmedHam(), isSpamOptionEnabled);
         SpamHandler spamHandler = isSpamOptionEnabled ? SpamHandlerRegistry.getSpamHandlerBySession(session, accountId) : NoSpamHandler.getInstance();
 
         // Collect SPECIAL-USE information
         TIntObjectMap<String> specialUseInfo = getSpecialUseInfo(names, fullNames);
-        TIntObjectMap<String> indexes = null;
-
+        boolean checkSpecialUseFolder = false;
         // Check if it is the first connect attempt for primary mail account
         if (MailAccount.DEFAULT_ID == accountId) {
-            boolean checkSpecialUseFolder = false;
+
             ConfigViewFactory viewFactory = Services.getService(ConfigViewFactory.class);
             if (viewFactory != null) {
                 ConfigView view = viewFactory.getView(session.getUserId(), session.getContextId());
                 ConfigProperty<Boolean> prop = view.property("user", "com.openexchange.mail.specialuse.check", Boolean.class);
+
                 if (prop.isDefined()) {
                     Boolean b = prop.get();
                     checkSpecialUseFolder = null != b && b.booleanValue();
                     prop.set(null);
                 }
             }
-
-            if (checkSpecialUseFolder) {
-                // Check for marked default folders on first connect attempt for primary mail account
-                indexes = specialUseInfo;
-            }
         }
 
-        // Sanitize given names and full-names against mail account settings
-        sanitizeAgainstMailAccount(names, fullNames, namespace, sep, indexes, accountChanged);
+        if (checkSpecialUseFolder) {
+            // Check for special use default folders on first connection attempt for primary mail account
+            sanitizeAgainstMailAccount(names, fullNames, defaultNames, namespace, sep, specialUseInfo, accountChanged);
+        } else {
+            sanitizeAgainstMailAccount(names, fullNames, defaultNames, namespace, sep, null, accountChanged);
+        }
 
         // Check folders
         TIntObjectMap<String> toSet = (MailAccount.DEFAULT_ID == accountId) ? null : new TIntObjectHashMap<String>(6);
@@ -650,9 +651,15 @@ public class IMAPDefaultFolderChecker {
                 }
 
                 // Check against account data
-                if ((null != toSet) && (null != checkedFullName) && (false == isEmpty(fullName)) && !checkedFullName.equals(fullName)) {
-                    toSet.put(index, checkedFullName);
-                    added = true;
+                if (null != toSet && (null != checkedFullName)) {
+                    if (isEmpty(fullName)) {
+                        // No full-name set before
+                        toSet.put(index, checkedFullName);
+                        added = true;
+                    } else if (!checkedFullName.equals(fullName)) {
+                        // Strange...
+                        LOG.warn("Invalid {} full-name in settings of external account for login {} (account={}) on IMAP server {}. Should be \"{}\", but is \"{}\" (user={}, context={})", getFallbackName(index), imapConfig.getLogin(), Integer.valueOf(accountId), imapConfig.getServer(), fullName, checkedFullName, Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
+                    }
                 }
             }
 
@@ -680,132 +687,78 @@ public class IMAPDefaultFolderChecker {
      *
      * @param names The names for standard folders
      * @param fullNames The full-names for standard folders
+     * @param defaultNames The fallback names
      * @param namespace The user's namespace; e.g. <code>"INBOX/"</code>
      * @param sep The separator character; e.g. <code>'/'</code>
-     * @param checkedIndexes The checked indexes according to SPECIAL-USE flags advertised by IMAP server (if any)
+     * @param specialUseInfo The optional full-names announced by the imap server
      * @param accountChanged The boolean reference to signal whether mail account has been changed
      */
-    protected void sanitizeAgainstMailAccount(String[] names, String[] fullNames, String namespace, char sep, TIntObjectMap<String> checkedIndexes, BoolReference accountChanged) {
-        // Special handling for full names in case of primary mail account
+    protected void sanitizeAgainstMailAccount(String[] names, String[] fullNames, String[] defaultNames, String namespace, char sep, TIntObjectMap<String> specialUseInfo, BoolReference accountChanged) {
         if (MailAccount.DEFAULT_ID == accountId) {
-            /*-
-             * Check full names for primary account:
-             *
-             * Null'ify full name if not on root level OR not equal to name; meaning not intended to create default folders next to INBOX
-             * In that case create them with respect to determined prefix
-             */
-            TIntObjectMap<String> namesToSet = new TIntObjectHashMap<String>(6);
-            TIntList indexes = new TIntLinkedList();
-            for (int i = 0; i < fullNames.length; i++) {
-                String fullName = fullNames[i];
-                if (isEmpty(fullName)) {
-                    // No full name given
-                    if (null != checkedIndexes) {
-                        String expectedFullName = checkedIndexes.get(i);
-                        if (null != expectedFullName) {
-                            // Deduce expected name from SPECIAL-USE full name
-                            String expectedName = expectedFullName.substring(expectedFullName.lastIndexOf(sep) + 1);
-                            if (!expectedName.equals(names[i])) {
-                                LOG.warn("Replacing invalid name in settings of primary account. Should be \"{}\", but is \"{}\" (user={}, context={})", expectedName, names[i], Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
-                                names[i] = expectedName;
-                                namesToSet.put(i, expectedName);
-                            }
-                        }
-                    }
-                    fullNames[i] = null;
-                } else {
-                    // Full name specified
-                    if (fullName.indexOf(sep) > 0 || !fullName.equals(names[i])) {
-                        // E.g. name=Sent, but fullName=INBOX/Sent or fullName=Zent
+            sanitizeAgainstPrimaryMailAccount(names, fullNames, defaultNames, namespace, sep, specialUseInfo, accountChanged);
+        }
+    }
 
-                        String expectedFullName = null;
-                        if (null != checkedIndexes) {
-                            expectedFullName = checkedIndexes.get(i);
-                        }
-                        if (null == expectedFullName) {
-                            // Deduce expected full-name from namespace + name concatenation
-                            expectedFullName = namespace + names[i];
-                            if (!expectedFullName.equals(fullName)) {
-                                LOG.warn("Clearing invalid full name in settings of primary account. Should be \"{}\", but is \"{}\" (user={}, context={})", expectedFullName, fullName, Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
-                                indexes.add(i);
-                            }
-                        } else {
-                            // Check against actual full-name
-                            if (!expectedFullName.equals(fullName)) {
-                                LOG.warn("Clearing invalid full name in settings of primary account. Should be \"{}\", but is \"{}\" (user={}, context={})", expectedFullName, fullName, Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
-                                indexes.add(i);
-                            }
-
-                            // Deduce expected name from SPECIAL-USE full name
-                            String expectedName = expectedFullName.substring(expectedFullName.lastIndexOf(sep) + 1);
-                            if (!expectedName.equals(names[i])) {
-                                LOG.warn("Replacing invalid name in settings of primary account. Should be \"{}\", but is \"{}\" (user={}, context={})", expectedName, names[i], Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
-                                names[i] = expectedName;
-                                namesToSet.put(i, expectedName);
-                            }
-                        }
+    private void sanitizeAgainstPrimaryMailAccount(String[] names, String[] fullNames, String[] defaultNames, String namespace, char sep, TIntObjectMap<String> specialUseInfo, BoolReference accountChanged) {
+        TIntObjectMap<String> namesToSet = new TIntObjectHashMap<>(6);
+        TIntObjectMap<String> fullNamesToSet = new TIntObjectHashMap<>(6);
+        for (int i = 0; i < fullNames.length; i++) {
+            String fullName = fullNames[i];
+            if (isEmpty(fullName)) {
+                if (!isEmpty(names[i])) {
+                    // Use only configured folder name.
+                    continue;
+                }
+                // No full-name or name given.
+                if (null != specialUseInfo) {
+                    String specialUseFullName = specialUseInfo.get(i);
+                    if (null != specialUseFullName) {
+                        // Update name and full-name with the full-name announced by the imap server.
+                        String specialUseName = specialUseFullName.substring(specialUseFullName.lastIndexOf(sep) + 1);
+                        fullNames[i] = specialUseFullName;
+                        fullNamesToSet.put(i, specialUseFullName);
+                        names[i] = specialUseName;
+                        namesToSet.put(i, specialUseName);
+                    } else {
+                        // Fall back to default name and update the accounts name.
                         fullNames[i] = null;
+                        names[i] = defaultNames[i];
+                        namesToSet.put(i, names[i]);
                     }
+                } else {
+                    // Fall back to default name and update the accounts name.
+                    fullNames[i] = null;
+                    names[i] = defaultNames[i];
+                    namesToSet.put(i, names[i]);
                 }
-            }
-            if (!indexes.isEmpty()) {
-                MailAccount modifiedAccount = clearAccountFullNames(indexes.toArray());
-                accountChanged.setValue(true);
-                if (null != modifiedAccount) {
-                    imapConfig.applyStandardNames(modifiedAccount, true);
-                }
-            }
-            if (!namesToSet.isEmpty()) {
-                MailAccount modifiedAccount = setAccountNames(namesToSet);
-                accountChanged.setValue(true);
-                if (null != modifiedAccount) {
-                    imapConfig.applyStandardNames(modifiedAccount, true);
-                }
-            }
-        } else {
-            if (null != checkedIndexes && !checkedIndexes.isEmpty()) {
-                TIntObjectMap<String> fullNamesToSet = new TIntObjectHashMap<String>(6);
-                TIntObjectMap<String> namesToSet = new TIntObjectHashMap<String>(6);
-                for (int i = 0; i < fullNames.length; i++) {
-                    String expectedFullName = checkedIndexes.get(i);
-                    if (null != expectedFullName) {
-                        String fullName = fullNames[i];
-                        if (isEmpty(fullName)) {
-                            fullNamesToSet.put(i, expectedFullName);
-                            // Check name, too
-                            String expectedName = expectedFullName.substring(expectedFullName.lastIndexOf(sep) + 1);
-                            if (!expectedName.equals(names[i])) {
-                                names[i] = expectedName;
-                                namesToSet.put(i, expectedFullName);
-                            }
-                        } else if (!expectedFullName.equals(fullName)) {
-                            fullNames[i] = null;
-                            fullNamesToSet.put(i, expectedFullName);
-                            // Check name, too
-                            String expectedName = expectedFullName.substring(expectedFullName.lastIndexOf(sep) + 1);
-                            if (!expectedName.equals(names[i])) {
-                                names[i] = expectedName;
-                                namesToSet.put(i, expectedFullName);
-                            }
+            } else {
+                // Full-name specified.
+                if (fullName.indexOf(sep) > 0 || !fullName.equals(names[i])) {
+                    String expectedName = fullName.substring(fullName.lastIndexOf(sep) + 1);
+                        if (!expectedName.equals(names[i])) {
+                        // Adapt configured name to configured full-name.
+                            LOG.warn("Replacing invalid name in settings of primary account. Should be \"{}\", but is \"{}\" (user={}, context={})", expectedName, names[i], Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
+                            names[i] = expectedName;
+                            namesToSet.put(i, expectedName);
                         }
-                    }
-                }
-                if (!fullNamesToSet.isEmpty()) {
-                    MailAccount modifiedAccount = setAccountFullNames(fullNamesToSet);
-                    accountChanged.setValue(true);
-                    if (null != modifiedAccount) {
-                        imapConfig.applyStandardNames(modifiedAccount, true);
-                    }
-                }
-                if (!namesToSet.isEmpty()) {
-                    MailAccount modifiedAccount = setAccountNames(namesToSet);
-                    accountChanged.setValue(true);
-                    if (null != modifiedAccount) {
-                        imapConfig.applyStandardNames(modifiedAccount, true);
-                    }
                 }
             }
-            LOG.debug("Checking standard folder for account {} (user={}, context={})", Integer.valueOf(accountId), Integer.valueOf(session.getUserId()), Integer.valueOf(session.getContextId()));
+        }
+        // Update account if necessary
+        if (!namesToSet.isEmpty()) {
+            MailAccount modifiedAccount = setAccountNames(namesToSet);
+            accountChanged.setValue(true);
+            if (null != modifiedAccount) {
+                imapConfig.applyStandardNames(modifiedAccount, true);
+            }
+        }
+
+        if (!fullNamesToSet.isEmpty()) {
+            MailAccount modifiedAccount = setAccountFullNames(fullNamesToSet);
+            accountChanged.setValue(true);
+            if (null != modifiedAccount) {
+                imapConfig.applyStandardNames(modifiedAccount, true);
+            }
         }
     }
 
@@ -972,7 +925,7 @@ public class IMAPDefaultFolderChecker {
         if (!f.exists()) {
             // Check against siblings
             IMAPFolder parent = (IMAPFolder) f.getParent();
-            List<Folder> candidates = new ArrayList<Folder>(2);
+            List<Folder> candidates = new ArrayList<>(2);
             {
                 Folder[] folders = parent.list();
                 String mName = f.getName();
@@ -1220,7 +1173,7 @@ public class IMAPDefaultFolderChecker {
         if (null != mass) {
             try {
                 TIntList indexes = new TIntArrayList(map.size());
-                List<String> fullNames = new ArrayList<String>(map.size());
+                List<String> fullNames = new ArrayList<>(map.size());
                 for (int index : map.keys()) {
                     indexes.add(index);
                     fullNames.add(map.get(index));
@@ -1246,7 +1199,7 @@ public class IMAPDefaultFolderChecker {
         if (null != mass) {
             try {
                 TIntList indexes = new TIntArrayList(map.size());
-                List<String> names = new ArrayList<String>(map.size());
+                List<String> names = new ArrayList<>(map.size());
                 for (int index : map.keys()) {
                     indexes.add(index);
                     names.add(map.get(index));
