@@ -651,7 +651,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
 
                     @Override
                     public void handle(final Header hdr, final IDMailMessage mailMessage) throws OXException {
-                        mailMessage.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(MimeMessageUtility.checkNonAscii(hdr.getValue())));
+                        mailMessage.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(MimeMessageUtility.checkNonAscii(hdr.getValue())), true);
                     }
                 });
                 put(MessageHeaders.HDR_DATE, new HeaderHandler() {
@@ -914,7 +914,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             msg.addReplyTo(wrap(env.replyTo));
             msg.setHeader("In-Reply-To", env.inReplyTo);
             msg.setHeader("Message-Id", env.messageId);
-            msg.setSubject(MimeMessageUtility.decodeEnvelopeSubject(env.subject));
+            msg.setSubject(MimeMessageUtility.decodeEnvelopeSubject(env.subject), true);
             msg.setSentDate(env.date);
         }
 
@@ -955,7 +955,7 @@ public final class MailMessageFetchIMAPCommand extends AbstractIMAPCommand<MailM
             }
             header = message.getHeader("Subject");
             if (null != header && header.length > 0) {
-                msg.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(header[0]));
+                msg.setSubject(MimeMessageUtility.decodeMultiEncodedHeader(header[0]), true);
             }
             msg.setSentDate(message.getSentDate());
         }
