@@ -136,10 +136,14 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
 
     private boolean propagateClientIPAddress;
 
+    private boolean useMultipleAddresses;
+
     private boolean enableTls;
 
     private boolean auditLogEnabled;
 
+    private boolean overwritePreLoginCapabilities;
+
     private Set<String> propagateHostNames;
 
     private boolean allowFolderCaches;
@@ -162,6 +166,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         sContainerType = "boundary-aware";
         enableTls = true;
         auditLogEnabled = false;
+        overwritePreLoginCapabilities = false;
         maxNumConnection = -1;
         newACLExtMap = new NonBlockingHashMap<String, Boolean>();
         mailProperties = MailProperties.getInstance();
@@ -219,7 +224,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         }
 
         {
-            final String imapSearchStr = configuration.getProperty("com.openexchange.imap.imapSearch", "imap").trim();
+            final String imapSearchStr = configuration.getProperty("com.openexchange.imap.imapSearch", "force-imap").trim();
             forceImapSearch = "force-imap".equalsIgnoreCase(imapSearchStr);
             imapSearch = forceImapSearch || "imap".equalsIgnoreCase(imapSearchStr);
             logBuilder.append("\tIMAP-Search: ").append(imapSearch).append(forceImapSearch ? " (forced)\n" : "\n");
@@ -237,6 +242,12 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             logBuilder.append("\tPropagate Client IP Address: ").append(propagateClientIPAddress).append('\n');
         }
 
+        {
+            String tmp = configuration.getProperty("com.openexchange.imap.useMultipleAddresses", STR_FALSE).trim();
+            useMultipleAddresses = Boolean.parseBoolean(tmp);
+            logBuilder.append("\tUse Multiple Addresses: ").append(useMultipleAddresses).append('\n');
+        }
+
         {
             final String tmp = configuration.getProperty("com.openexchange.imap.enableTls", STR_TRUE).trim();
             enableTls = Boolean.parseBoolean(tmp);
@@ -249,6 +260,12 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             logBuilder.append("\tAudit Log Enabled: ").append(auditLogEnabled).append('\n');
         }
 
+        {
+            String tmp = configuration.getProperty("com.openexchange.imap.overwritePreLoginCapabilities", STR_FALSE).trim();
+            overwritePreLoginCapabilities = Boolean.parseBoolean(tmp);
+            logBuilder.append("\tOverwrite Pre-Login Capabilities: ").append(overwritePreLoginCapabilities).append('\n');
+        }
+
         {
             final String tmp = configuration.getProperty("com.openexchange.imap.propagateHostNames", "").trim();
             if (tmp.length() > 0) {
@@ -436,7 +453,7 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
             this.cipherSuites = Strings.isEmpty(tmp) ? null : tmp;
             logBuilder.append("\tSupported SSL cipher suites: ").append(null == this.cipherSuites ? "<default>" : cipherSuites).append("\n");
         }
-        
+
         {
             String tmp = configuration.getProperty("com.openexchange.imap.greeting.host.regex", "").trim();
             tmp = Strings.isEmpty(tmp) ? null : tmp;
@@ -463,8 +480,10 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         forceImapSearch = false;
         fastFetch = true;
         propagateClientIPAddress = false;
+        useMultipleAddresses = false;
         enableTls = true;
         auditLogEnabled = false;
+        overwritePreLoginCapabilities = false;
         propagateHostNames = Collections.emptySet();
         supportsACLs = null;
         imapTimeout = 0;
@@ -520,6 +539,11 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return auditLogEnabled;
     }
 
+    @Override
+    public boolean isOverwritePreLoginCapabilities() {
+        return overwritePreLoginCapabilities;
+    }
+
     @Override
     public Set<String> getPropagateHostNames() {
         return propagateHostNames;
@@ -722,4 +746,13 @@ public final class IMAPProperties extends AbstractProtocolProperties implements
         return hostExtractingGreetingListener;
     }
 
+    /**
+     * Checks whether possible multiple IP addresses for a host name are supposed to be considered.
+     *
+     * @return <code>true</code> to use multiple IP addresses; otherwise <code>false</code>
+     */
+    public boolean isUseMultipleAddresses() {
+        return useMultipleAddresses;
+    }
+
 }
