@@ -51,6 +51,7 @@ package com.openexchange.imap.storecache;
 
 import java.util.AbstractQueue;
 import java.util.Collection;
+import java.util.Collections;
 import java.util.Iterator;
 import java.util.NoSuchElementException;
 import java.util.PriorityQueue;
@@ -61,8 +62,11 @@ import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.ReentrantLock;
 import javax.mail.MessagingException;
 import com.openexchange.imap.IMAPClientParameters;
+import com.openexchange.imap.config.IMAPProperties;
 import com.openexchange.log.LogProperties;
+import com.openexchange.mailaccount.MailAccount;
 import com.openexchange.session.Session;
+import com.sun.mail.imap.GreetingListener;
 import com.sun.mail.imap.IMAPStore;
 
 /**
@@ -83,8 +87,8 @@ public class UnboundedIMAPStoreContainer extends AbstractIMAPStoreContainer {
     /**
      * Initializes a new {@link UnboundedIMAPStoreContainer}.
      */
-    public UnboundedIMAPStoreContainer(String server, int port, boolean propagateClientIp, boolean checkConnectivityIfPolled) {
-        super(propagateClientIp);
+    public UnboundedIMAPStoreContainer(int accountId, Session session, String server, int port, boolean propagateClientIp, boolean checkConnectivityIfPolled) {
+        super(accountId, session, propagateClientIp);
         maxRetryCount = 10;
         availableQueue = new InheritedPriorityBlockingQueue();
         this.port = port;
@@ -133,9 +137,16 @@ public class UnboundedIMAPStoreContainer extends AbstractIMAPStoreContainer {
         }
 
         // Grab associated IMAP session identifier (as advertised via "ID" command)
-        String sessionInformation = imapStore.getClientParameter(IMAPClientParameters.SESSION_ID.getParamName());
+        String sessionInformation = imapStore.getGeneratedExternalId();
         if (null != sessionInformation) {
             LogProperties.put(LogProperties.Name.MAIL_SESSION, sessionInformation);
+            if (accountId == MailAccount.DEFAULT_ID) {
+                GreetingListener greetingListener = IMAPProperties.getInstance().getHostNameRegex();
+                if (null != greetingListener) {
+                    String greeting = imapStore.getGreeting();
+                    greetingListener.onGreetingProcessed(greeting, imapStore.getHost(), imapStore.getPort());
+                }
+            }
         }
 
         // Should we set properties from passed session?
