@@ -52,6 +52,7 @@ package com.openexchange.imap.threadsort2;
 import static com.openexchange.imap.util.ImapUtility.prepareImapCommandForLogging;
 import static com.openexchange.java.Strings.isDigit;
 import static com.openexchange.mail.MailServletInterface.mailInterfaceMonitor;
+import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.Comparator;
@@ -59,21 +60,26 @@ import java.util.LinkedList;
 import java.util.List;
 import javax.mail.FetchProfile;
 import javax.mail.MessagingException;
+import javax.mail.search.SearchException;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPException;
 import com.openexchange.imap.IMAPServerInfo;
 import com.openexchange.imap.command.MailMessageFillerIMAPCommand;
 import com.openexchange.imap.threader.references.Conversation;
+import com.openexchange.imap.util.WrappingProtocolException;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.MailField;
 import com.openexchange.mail.dataobjects.IDMailMessage;
 import com.openexchange.mail.dataobjects.MailMessage;
 import com.openexchange.mail.mime.utils.MimeStorageUtility;
+import com.openexchange.mail.search.SearchTerm;
+import com.sun.mail.iap.Argument;
 import com.sun.mail.iap.ProtocolException;
 import com.sun.mail.iap.Response;
 import com.sun.mail.imap.IMAPFolder;
 import com.sun.mail.imap.protocol.IMAPProtocol;
 import com.sun.mail.imap.protocol.IMAPResponse;
+import com.sun.mail.imap.protocol.SearchSequence;
 
 
 /**
@@ -101,19 +107,33 @@ public final class ThreadSorts {
      * @return The thread-sort string.
      * @throws MessagingException If a messaging error occurs
      */
-    public static String getThreadResponse(final IMAPFolder imapFolder, final String sortRange, final boolean uid) throws MessagingException {
+    public static String getThreadResponse(final IMAPFolder imapFolder, final String sortRange, final boolean uid, final SearchTerm<?> searchTerm) throws MessagingException {
         final org.slf4j.Logger log = LOG;
         final Object val = imapFolder.doCommand(new IMAPFolder.ProtocolCommand() {
 
             @Override
             public Object doCommand(final IMAPProtocol p) throws ProtocolException {
-                final String command = new StringBuilder(32).append(uid ? "UID THREAD" : "THREAD").append(" REFERENCES UTF-8 ").append(sortRange).toString();
+                String cmd = uid ? "UID THREAD" : "THREAD";
+
+                Argument args = new Argument().writeAtom("REFERENCES UTF-8");
+
+                if (searchTerm != null) {
+                    try {
+                        args.append(new SearchSequence().generateSequence(searchTerm.getJavaMailSearchTerm(), "UTF-8"));
+                    } catch (final IOException ioex) {
+                        // should never happen
+                        throw new WrappingProtocolException("", new SearchException(ioex.toString()));
+                    } catch (MessagingException e) {
+                        throw new WrappingProtocolException("", e);
+                    }
+                }
+                args.writeAtom(sortRange);
                 final Response[] r;
                 {
                     final long start = System.currentTimeMillis();
-                    r = p.command(command, null);
+                    r = p.command(cmd, args);
                     final long dur = System.currentTimeMillis() - start;
-                    log.debug("\"{}\" for \"{}\" ({}) took {}msec.", command, imapFolder.getFullName(), imapFolder.getStore(), Long.valueOf(dur));
+                    log.debug("\"{}\" for \"{}\" ({}) took {}msec.", cmd, imapFolder.getFullName(), imapFolder.getStore(), Long.valueOf(dur));
                     mailInterfaceMonitor.addUseTime(dur);
                 }
                 final Response response = r[r.length - 1];
@@ -132,13 +152,13 @@ public final class ThreadSorts {
                     }
                     p.notifyResponseHandlers(r);
                 } else if (response.isBAD()) {
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(command));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(cmd));
                     throw new ProtocolException(new StringBuilder("IMAP server does not support THREAD command: ").append(response.toString()).toString());
                 } else if (response.isNO()) {
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(command));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(cmd));
                     throw new ProtocolException(new StringBuilder("IMAP server does not support THREAD command: ").append(response.toString()).toString());
                 } else {
-                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(command));
+                    LogProperties.putProperty(LogProperties.Name.MAIL_COMMAND, prepareImapCommandForLogging(cmd));
                     p.handleResult(response);
                 }
                 return retval;
@@ -161,9 +181,9 @@ public final class ThreadSorts {
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, IMAPServerInfo serverInfo, MailField... fields) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(IMAPFolder imapFolder, String sortRange, boolean isRev1, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm, MailField... fields) throws OXException, MessagingException {
         if (null == fields || fields.length == 0 || (fields.length == 1 && MailField.RECEIVED_DATE.equals(fields[0]))) {
-            return parseConversations(getThreadResponse(imapFolder, sortRange, true), imapFolder.getFullName());
+            return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
 
         // Its fetch profile...
@@ -171,7 +191,7 @@ public final class ThreadSorts {
         for (MailField field : fields) {
             MimeStorageUtility.addFetchItem(fp, field);
         }
-        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo);
+        return getConversations(imapFolder, sortRange, isRev1, fp, serverInfo, searchTerm);
     }
 
     /**
@@ -188,16 +208,16 @@ public final class ThreadSorts {
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<List<MailMessage>> getConversations(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo) throws OXException, MessagingException {
+    public static List<List<MailMessage>> getConversations(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm) throws OXException, MessagingException {
         if (null == fetchProfile) {
-            return parseConversations(getThreadResponse(imapFolder, sortRange, true), imapFolder.getFullName());
+            return parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         }
 
-        List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true), imapFolder.getFullName());
+        List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
 
         {
             // Turn conversations to a flat list
-            List<MailMessage> msgs = new ArrayList<MailMessage>(conversations.size() << 2);
+            List<MailMessage> msgs = new ArrayList<>(conversations.size() << 2);
             for (List<MailMessage> conversation : conversations) {
                 for (MailMessage m : conversation) {
                     msgs.add(m);
@@ -224,11 +244,11 @@ public final class ThreadSorts {
      * @throws OXException If parsing fails
      * @throws MessagingException If acquiring the THREAD response fails
      */
-    public static List<Conversation> getConversationList(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo) throws OXException, MessagingException {
-        List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true), imapFolder.getFullName());
+    public static List<Conversation> getConversationList(final IMAPFolder imapFolder, final String sortRange, final boolean isRev1, final FetchProfile fetchProfile, IMAPServerInfo serverInfo, SearchTerm<?> searchTerm) throws OXException, MessagingException {
+        List<List<MailMessage>> conversations = parseConversations(getThreadResponse(imapFolder, sortRange, true, searchTerm), imapFolder.getFullName());
         {
             // Turn conversations to a flat list
-            List<MailMessage> msgs = new ArrayList<MailMessage>(conversations.size() << 2);
+            List<MailMessage> msgs = new ArrayList<>(conversations.size() << 2);
             for (List<MailMessage> conversation : conversations) {
                 for (MailMessage m : conversation) {
                     msgs.add(m);
@@ -238,7 +258,7 @@ public final class ThreadSorts {
             new MailMessageFillerIMAPCommand(msgs, isRev1, fetchProfile, serverInfo, imapFolder).doCommand();
         }
 
-        List<Conversation> retval = new ArrayList<Conversation>(conversations.size());
+        List<Conversation> retval = new ArrayList<>(conversations.size());
         for (List<MailMessage> conversationMessages : conversations) {
             retval.add(new Conversation(conversationMessages));
         }
@@ -256,11 +276,15 @@ public final class ThreadSorts {
      * @throws OXException If parsing fails
      */
     public static List<List<MailMessage>> parseConversations(String threadList, String fullName) throws OXException {
-        List<List<MailMessage>> conversations = new LinkedList<List<MailMessage>>();
+        List<List<MailMessage>> conversations = new LinkedList<>();
 
         int length = threadList.length();
         int off = threadList.indexOf('(');
 
+        if (length == 0 || off == -1) {
+            return conversations;
+        }
+
         while (off < length) {
             char c = threadList.charAt(off);
             if (c != '(') {
@@ -313,7 +337,7 @@ public final class ThreadSorts {
         int off = 0;
 
         StringBuilder digits = new StringBuilder(8);
-        List<IDMailMessage> list = new LinkedList<IDMailMessage>();
+        List<IDMailMessage> list = new LinkedList<>();
 
         while (off < length) {
             char c = conversation.charAt(off++);
